package rdbtoonto.dbmodel;


import java.io.*;
import java.lang.*;
import java.util.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

/**
 * @author  scdtncf
 */
public class MSA_RDBFormHandler extends DefaultHandler {
	protected DBSchema _DBSchema;
	protected String _CurrEltName;
	protected String _LastElt;
	protected TableDef _CurrTable;
	
	protected String _Cont;
	protected Hashtable<String,String> _GraphAtts;
	protected FormDetails _FormDetails;
	protected Hashtable<String,Hashtable<String,String>> _FriendlyNames; // (field, textfield) couples
	public MSA_RDBFormHandler(DBSchema rdbs) {
		_DBSchema = rdbs;	
	}
	
	public DBSchema getRDBSchema(){
		return _DBSchema;
	}
	
	public void startDocument() {
		_FormDetails = new FormDetails();
		_FriendlyNames = new Hashtable<String, Hashtable<String,String>>();
		_LastElt="";
	}
	
	public void endDocument() {
		if (isTabularPresentation()) {
			extractTabularAssociations();
		}
		else {
			extractFormAssociations();
		}
		addALLFriendlyNames();
		
		
		Set<String> keys = _FriendlyNames.keySet();
	/*	System.out.println("INTER FRIENDLY");
			for(String field : keys) {
				System.out.println(field + ":" + _FriendlyNames.get(field).get("textfield"));
		}
		*/
	}
	
	public boolean isTabularPresentation() {
		
		if (hasAlignedTextsAndFields()) {
			return false;
		}
		String prevTop = _FormDetails._FieldTexts.elementAt(0).get("TOP");
		String currTop;
		int nbAligned = 1;
		for(int i=1;i < _FormDetails._FieldTexts.size();i++) {
			currTop = _FormDetails._FieldTexts.elementAt(i).get("TOP");
			if (currTop.equals(prevTop)) {
				nbAligned++;
			}
		}
		if (nbAligned > _FormDetails._FieldTexts.size() - 2) {
			return true;
		}
		else {
			return false;
		}
	}
	
	
	public boolean hasAlignedTextsAndFields() {
		
		double currTTop;
		double currFTop;
		
		for(int i=1;i < _FormDetails._FieldTexts.size();i++) {
			currTTop = getNumValueOfAtt(_FormDetails._FieldTexts.elementAt(i).get("TOP"));

			for(int j=1;j < _FormDetails._Fields.size();j++) {
				currFTop = getNumValueOfAtt(_FormDetails._Fields.elementAt(j).get("TOP"));
				
				if (areClose(currTTop,currFTop)) {
					return true;
				}
			}
		}
		return false;
	}
		
	public void extractTabularAssociations() {
	//	System.out.println("This form is tabular: " + _FormDetails._FieldTexts.size());
		String leftAtt;
		for(Hashtable<String,String> spanAtts : _FormDetails._FieldTexts) {
			leftAtt = spanAtts.get("LEFT");
			String closestField = getHClosest(leftAtt);
			if (closestField !="") {
				_FriendlyNames.put(closestField,
						spanAtts);
			}
		}
	}
	
	public String getHClosest(String leftAtt) {
		String closestField="";
		
		double left_Att = Double.parseDouble(leftAtt.substring(0,leftAtt.length() -2));
		double curr_left=0;
		double maxDelta = 1000;
		double delta = 1000;
		String curr_left_str;
		Hashtable<String,String> closestAtts = null;
		
		for(Hashtable<String,String> spanAtts : _FormDetails._Fields) {
			curr_left_str = spanAtts.get("LEFT");
			curr_left_str = curr_left_str.substring(0,curr_left_str.length()-2);
			curr_left = Double.parseDouble(curr_left_str);
			delta = left_Att - curr_left;
			if (delta < 0) {
				delta = -delta;
			}
			
			if (delta < maxDelta) {
				closestField = spanAtts.get("field");
				maxDelta = delta;
				closestAtts = spanAtts;
			}
		}
		if (closestAtts != null) {
			_FormDetails._Fields.remove(closestAtts);		}
		return closestField;
	}
	
	
	public Vector<Hashtable<String,String>> getSpansWithV(double top) {
		Vector<Hashtable<String,String>> spans = new Vector<Hashtable<String,String>>();
		double curr_top;
		
		for(Hashtable<String,String> spanAtts : _FormDetails._Fields) {	
			curr_top = getNumValueOfAtt(spanAtts.get("TOP"));
			
			if (areClose(top,curr_top)) {
				spans.add(spanAtts);
			}
		}
		return spans;
	}
	
	public Hashtable<String,String> getHClosestOnSameV(String leftText, String topText) {
		String closestField="";
		
		double left_Text = getNumValueOfAtt(leftText);
		double left_Field=0;
		double maxDelta = 1000;
		double delta = 1000;
		double top_Text= getNumValueOfAtt(topText);
		Hashtable<String,String> closestAtts = null;
		
		Vector<Hashtable<String,String>> spans = getSpansWithV(top_Text);		
		for(Hashtable<String,String> spanAtts : spans) {
			left_Field = getNumValueOfAtt(spanAtts.get("LEFT"));
			delta = left_Field - left_Text;
			// System.out.println("->TEST field=" + spanAtts.get("field") + ", top=" + top_Text + ", left=" + left_Field);
		
			if ((delta > 0) && (delta < maxDelta)) {
				closestField = spanAtts.get("field");
				maxDelta = delta;
				closestAtts = spanAtts;
			}
		}
	//	System.out.println("Out of loop:" + closestAtts);
		if (closestAtts != null) {
			//	_FormDetails._Fields.remove(closestAtts);
	//		System.out.println("-> SELECTED field=" + closestField + ":" + closestAtts.get("field") + ", top=" + closestAtts.get("TOP") + ", left=" + left_Field);
		}
		return closestAtts;
	}
	
	public boolean areClose(double top1,double top2) {
		
		double threshold = 0.05; // in cm
		double gap = top1 - top2;
		if (gap < 0) {
			gap = -gap;
		}
		
		if (gap <= threshold) {
			
			return true;
		}
		else {
			return false;
		}	
	}
	
	public void extractFormAssociations() {
		// System.out.println("This form is NOT tabular");
		String leftAtt;
		String topAtt;
	//	System.out.println("Nb text fields to scan:" + _FormDetails._FieldTexts.size());
		Hashtable<String,String> closestFieldSpan;
		for(Hashtable<String,String> spanAtts : _FormDetails._FieldTexts) {
			leftAtt = spanAtts.get("LEFT");
			topAtt = spanAtts.get("TOP");
	//		System.out.println("Textfield=" + spanAtts.get("textfield") + ", top=" + topAtt + ", left=" + leftAtt);
			closestFieldSpan = getHClosestOnSameV(leftAtt,topAtt);
			
			if (closestFieldSpan != null) {
				updateFriendlyName(closestFieldSpan,
						spanAtts);
			}
		}
	}
	
	public void updateFriendlyName(Hashtable<String,String> fieldAtts, Hashtable<String,String> nameAtts) {
		String fieldExpr =fieldAtts.get("field");
		
		String field = extractFieldName(fieldExpr);
		
		
		Hashtable<String,String> existingName = _FriendlyNames.get(field);
		
		if (existingName == null) {
			_FriendlyNames.put(field,nameAtts);
		}
		else {
			double fieldLeft = getNumValueOfAtt(fieldAtts.get("LEFT"));
			double existingLeft = getNumValueOfAtt(existingName.get("LEFT"));
			double currLeft =  getNumValueOfAtt(nameAtts.get("LEFT"));
			
			double delta1 = fieldLeft - existingLeft;
			if (delta1 < 0) delta1 = -delta1;
			
			double delta2 = fieldLeft - currLeft;
			if (delta2 < 0) delta2 = -delta2;
			
			if (delta2 < delta1) {
		//		System.out.println("Updated + closer: " + fieldAtts.get("field") + "--->" + nameAtts.get("textfield") + "(" + existingName.get("LEFT") + ")");
				_FriendlyNames.put(field,nameAtts);
			}
		}				
	}
	
	public String extractFieldName(String fieldExpr) {
//		System.out.println("fieldExpr=" + fieldExpr);
		if (fieldExpr.startsWith("fx:")) {
			String field = "";		
			int pos = 18;
			char carcour = fieldExpr.charAt(pos);
			while(carcour != '\'') {
				field += carcour;
				pos++;
				carcour = fieldExpr.charAt(pos);
			}
	//		System.out.println("field=" + field);
			return field;
		}
		else {
			return fieldExpr;
		}
	}
	
	
	public double getNumValueOfAtt(String numStr) {
		numStr = numStr.substring(0,numStr.length()-2);
		double num = Double.parseDouble(numStr);
		return num;
	}
	
	public void addALLFriendlyNames() {
		
		Set<String> keys = _FriendlyNames.keySet();
		
		if (_CurrTable == null) {	
			for(String field : keys) {
				_DBSchema.addFriendlyName(field,_FriendlyNames.get(field).get("textfield"));
			}
		}
		else {
			Attribute att;
			for(String field : keys) {
				att = _CurrTable.getAttribute(field);
				att.addFriendlyName(_FriendlyNames.get(field).get("textfield"));
			}
		}
	}
	
	public void startElement(String uri,
			String localName,
			String qname,
			Attributes attl) {
		
		_Cont ="";
		//	System.out.println("elt:" + localName + " (" + _LastElt + ")");
		
		if ((_LastElt.equals("body")) && (localName.equals("variable"))) {
			String select = attl.getValue("select");
			_CurrTable = _DBSchema.getTableDef(select);
		}
		
		if (localName.equals("span")) {
			String style = attl.getValue("style");
			_GraphAtts = getGraphAtts(style);
		}
		else if (_LastElt.equals("span")) {
			if (localName.equals("value-of")) {
				_GraphAtts.put("field",attl.getValue("select"));
				_FormDetails._Fields.add(_GraphAtts);
				_GraphAtts = null;
			}
		}
		_LastElt = localName;
	}
	
	public Hashtable<String,String> getGraphAtts(String style) {
		Hashtable<String,String> attvals = new Hashtable<String,String>();
		StringTokenizer atts = new StringTokenizer(style,";");
		String attval;
		while(atts.hasMoreElements()) {
			attval = atts.nextToken();
			StringTokenizer toks = new StringTokenizer(attval,":");
			attvals.put(toks.nextToken().trim(),toks.nextToken().trim());
		}
		return attvals;
	}
	
	public void endElement(String uri,
			String localName,
			String qname) {
		
		if ((localName.equals("span")) && 
				(_GraphAtts != null) && 
				(_Cont !="")) {
			_Cont = _Cont.trim();
			if (_Cont.endsWith(":")) {
				_Cont = _Cont.substring(0,_Cont.length() - 1);
				_Cont = _Cont.trim();
			}
			_GraphAtts.put("textfield",_Cont);
			_FormDetails._FieldTexts.add(_GraphAtts);
		}
	}
	
	public void characters(char ch[], int start, int length) {
		
		//      if (_CurrEltName.equals("TERM")) {
		
		int end = start + length;
		
		for(int i = start; i < end ; i++) {
			_Cont += ch[i];   
		}
		//  System.out.println("Lemme: " + _Lemme + " (" + start + ", " + length +")");
		// }
	}
	
	/*-----------------------------------------------------------------*/
	/* Implmentations des mthodes de traitement d'erreurs             */
	/*-----------------------------------------------------------------*/
	
	public void warning(SAXParseException e) throws SAXException {
		System.err.println("Warning: line : " +  e.getLineNumber() + ", col : " + e.getColumnNumber() + " : " + e.getMessage());
	}
	
	public void error(SAXParseException e) throws SAXException {
		System.err.println("Error !  line : " + e.getLineNumber() + ", col " + e.getColumnNumber() + " : " + e.getMessage());
	}
	
	public void fatalError(SAXParseException e) throws SAXException {
		System.err.println("Fatal Error ! line : " + e.getLineNumber() + ", col " + e.getColumnNumber() + " : " + e.getMessage());
	}
}


