package rdbtoonto.dbmodel;


import java.io.*;
import java.lang.*;
import java.util.*;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import rdbtoonto.RDBToOnto;
import rdbtoonto.utils.Utilities;

/**
 * @author  scdtncf
 */
public class MSA_RDBSchemaHandler extends DefaultHandler {
    protected DBSchema _RDBSchema;
    protected String _CurrEltName;
    protected String _LastElt;
    protected TableDef _TableDef;
    
    public MSA_RDBSchemaHandler(DBSchema rdbs) {
    	_RDBSchema = rdbs;
    }

  public DBSchema getDBSchema(){
    return _RDBSchema;
  }

  public void startDocument() {
    }

  public void endDocument() {
	  _TableDef = null;
  }

  public void startElement(String uri,
                           String localName,
			   String qname,
	                   Attributes attl) {

	    if (localName.equals("element")) {
	    	_LastElt = attl.getValue("name");
	    }
	    if (localName.equals("index")) {
	    	if (attl.getValue("primary").equals("yes")) {
	    		StringTokenizer keys = new StringTokenizer(attl.getValue("index-key").trim()," ");
	    		
	    		Vector<String> keyVects = new Vector<String>();
	    		
	    		while(keys.hasMoreTokens()) {
	    			keyVects.add(Utilities.normaliseName(keys.nextToken()));
	    		}
	    		PrimaryKey pkey = new PrimaryKey(keyVects);
	    		_TableDef.setPrimaryKey(pkey);
	    	}
	    _LastElt = attl.getValue("name");
	    }
	    if (localName.equals("annotation")) {
	    	RDBToOnto.highPrintln("Creating table: " + _LastElt + " ...");
	    	String tname = Utilities.normaliseName(_LastElt);
	    	_TableDef = new TableDef(tname);	
	    	_RDBSchema.addTableDef(_TableDef);
	    }
	    else if ((localName.equals("element")) && (_TableDef != null)) {

	    	String baseName = Utilities.normaliseName(attl.getValue("name"));
	    	RDBToOnto.highPrintln("Creating attribute: " + 
	    			baseName + " ...");
	    	String txtType = attl.getValue("od:jetType");
	    	AType atype;
	    	if ((txtType.equals("text")) || (txtType.equals("memo"))) {
	    		 atype = AType.XSD_STRING;
	    	}
	    	else if ((txtType.equals("int")) || (txtType.equals("longinteger"))) {
	    		 atype = AType.XSD_INT;
	    	}
	    	else if (txtType.equals("double"))  {
	    		 atype = AType.XSD_DOUBLE;
	    	}
	    	else if (txtType.equals("yesno"))  {
	    		 atype = AType.XSD_BOOLEAN;
	    	}
	    	else {
	    	  atype = AType.XSD_STRING;
	    	}
	    	Attribute natt = new Attribute(baseName,atype);
	    	_TableDef.addAttribute(natt);
	    	
	    	if (_TableDef.hasPrimaryKey(natt.getName())) {
	    		natt.setIsPrimaryKey(true);
	    	}
	    	
	    }
  }

  public void endElement(String uri,
			 String localName,
			   String qname) {
      if (localName.equals("schema")) {
    	  if (_TableDef.hasPrimaryKey()) {
    	  StringTokenizer keys = new StringTokenizer(_TableDef.getPrimaryKey().getAttribute());
    	  if (keys.countTokens() > 1) {
    		  boolean foundKey = false;
    		  String aKey;
    		  while ((keys.hasMoreTokens()) && !foundKey) {
    			  aKey = keys.nextToken();
    			  if (_TableDef.hasAttribute(aKey)) {
    				  foundKey = true;
    				  _TableDef.setPrimaryKey(new PrimaryKey(aKey));
    			  }
    		  }
    	  }
    	  }
      }
   }

	/*-----------------------------------------------------------------*/
	/* Error handling         */
	/*-----------------------------------------------------------------*/
	
	public void warning(SAXParseException e) throws SAXException {
		System.err.println("Warning ! line: " +  e.getLineNumber() + ", col : " + e.getColumnNumber() + " : " + e.getMessage());
	}
	
	public void error(SAXParseException e) throws SAXException {
		System.err.println("Error !  line: " + e.getLineNumber() + ", col " + e.getColumnNumber() + " : " + e.getMessage());
	}
	
	public void fatalError(SAXParseException e) throws SAXException {
		System.err.println("Fatal Error  ! line: " + e.getLineNumber() + ", col " + e.getColumnNumber() + " : " + e.getMessage());
	}
	
 }


