
/**
 * TAO - RDBToOnto
 * 
 * Schema Reader for MS Access XML database exports
 * 
 */


package rdbtoonto.dbmodel;


// Imported JAXP classes

import org.xml.sax.InputSource;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.XMLReader;

import rdbtoonto.RDBToOnto;
import rdbtoonto.utils.Utilities;

// Imported java.io classes
import java.io.BufferedReader;
import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.InputStreamReader;

public class XMLAccess_Reader extends DBReader {
    protected File[] _XSDFiles;
    protected File[] _XSLFiles;
    protected String _DataDir;
    
    public XMLAccess_Reader(String dbDescrFile, String user, String password) throws DBReaderException {
	super(dbDescrFile, user, password);
    try {

        BufferedReader in = new BufferedReader(
    					   new InputStreamReader(
    						 new FileInputStream(dbDescrFile)));

      in.readLine(); // header
      _DataDir = in.readLine(); // name of directory where data exports are stored
      
	File dirFile = new File(_DataDir);
	
	if ((!dirFile.exists()) || (!dirFile.isDirectory())) {
		System.err.println("RDBToOnto Error: XMLAccess_Reader: The provided input database does not exist or is not a directory: " + dirFile);
		throw new DBReaderException("RDBToOnto Error: MSA_XMLReader. Bad directory provided for input database");
	}
	
	File dir = new File(_DataDir);
	
		_XSDFiles = dir.listFiles(new FilenameFilter(){
			public boolean accept(File dir, String name) {
				if (name.endsWith(".xsd")) {
					return true;
				}
				else {
					return false;
				}
			}
		});
	
		File dirForms = new File(_DataDir + File.separator + "FORMS");
		
		_XSLFiles = dirForms.listFiles(new FilenameFilter(){
			public boolean accept(File dir, String name) {
				if (name.endsWith(".xsl")) {
					return true;
				}
				else {
					return false;
				}
			}
		});
    }
    catch(IOException ef) {
		System.err.println("RDBToOnto Error: XMLAccess_Reader: The provided input database does not exist or is not a directory: " + dbDescrFile);
		throw new DBReaderException("RDBToOnto Error: XMLAccess_Reader. Bad database descriptor file provided for input database");
    }
  }
    
    
    public boolean readSchema() {
    	RDBToOnto.lowPrintln("Reading database schema ...");
    	boolean result = true;
    	XMLReader reader=null;
    	InputSource src;
    	try {
	    System.setProperty("org.xml.sax.driver",
	                   "org.apache.xerces.parsers.SAXParser");
	    
	     reader = XMLReaderFactory.createXMLReader();
	     
    reader.setFeature("http://xml.org/sax/features/validation", false);
	
	_Database._DBSchema = new DBSchema();
    
    MSA_RDBSchemaHandler handler = new MSA_RDBSchemaHandler(_Database._DBSchema);

    reader.setContentHandler(handler);

    }
    catch (Exception e) {
            System.err.println("RDBToOnto:  pbm with schema reader Init.: " + e.getMessage());
            return false;
     }
    
	RDBToOnto.highPrintln("Reading RDB description from:" + _Loc);

	RDBToOnto.highPrintln(" - " + _XSDFiles.length + " to parse ...");
 
	for(File fileN : _XSDFiles) {
		String name = fileN.getName();
		if ((!(name.startsWith("~TMP"))) && // To avoid extra MSA generated files
			(!(name.startsWith("MSys"))) &&
			(!(name.startsWith("Table des erreurs")))
		    )
		{
			try {
			RDBToOnto.highPrintln("f:" + fileN);
			src = new InputSource(new FileInputStream(fileN));
	        reader.parse(src);
			  }
		      catch (Exception e) {
		              System.err.println("RDBToOnto:  DB Schema Reader:parsing: " + fileN.getName()+ ":" + e.getMessage());
		              return false;
		       }
	   }
	}
	result = readRelations();
	if (result) {
	scanFormsForFriendlyNames();
	}
	
	return result;
  }
    
    public boolean readRelations() {
    	try {

        	InputSource src;
    	    System.setProperty("org.xml.sax.driver",
    	                   "org.apache.xerces.parsers.SAXParser");
    	    XMLReader reader = XMLReaderFactory.createXMLReader();

        reader.setFeature("http://xml.org/sax/features/validation", false);
    	
        MSA_RDBRelationHandler handler = new MSA_RDBRelationHandler(_Database._DBSchema);

        reader.setContentHandler(handler);

    	String rfile = _DataDir + File.separator + "RelationsTable.xml";

    	RDBToOnto.highPrintln("Reading relations in " + rfile + "...");

		src = new InputSource(new FileInputStream(new File(rfile)));
        reader.parse(src);
        return true;
          }
          catch (Exception e) {
                  System.err.println("RDBToOnto:  DB Relation Reader:parse: " + e.getMessage());
	              return false;
              }
      }
    
    public void scanFormsForFriendlyNames() {
    	try {

        	InputSource src;
    	    System.setProperty("org.xml.sax.driver",
    	                   "org.apache.xerces.parsers.SAXParser");
    	    XMLReader reader = XMLReaderFactory.createXMLReader();

        reader.setFeature("http://xml.org/sax/features/validation", false);
    	
        MSA_RDBFormHandler handler = new MSA_RDBFormHandler(_Database._DBSchema);

        reader.setContentHandler(handler);

    	RDBToOnto.highPrintln("Reading XSL forms from ...");

    	RDBToOnto.highPrintln(" - " + _XSLFiles.length + " to parse ...");
     
    	for(File fileN : _XSLFiles) {
    			RDBToOnto.highPrintln("Parsing form file:" + fileN);
    			src = new InputSource(new FileInputStream(fileN));
    	        reader.parse(src);
    	}
  
          }
          catch (Exception e) {
                  System.err.println("RDBToOnto Warning:  DB Form Reader:parse: " + e.getMessage());
              }
      }

    // resultData fails if we've been unable to read even one table.
    
    public boolean readData() { 
    	boolean result = false;
    	for(TableDef td : _Database._DBSchema.getTableDefs()) {
    		result = readDataTable(td);
    	}
    	return result;
    }
    
  public boolean readDataTable(TableDef td) {	
 	try {
 		 RDBToOnto.highPrintln("Reading table data:" + td._Name + "...");
    	InputSource src;
    	String tabname = getFileNameFromTableDefName(td._Name);
    	String fileName = _DataDir + File.separator + tabname +".xml";
    	
    	  System.setProperty("org.xml.sax.driver",
    	                   "org.apache.xerces.parsers.SAXParser");
    	    XMLReader reader = XMLReaderFactory.createXMLReader();

        reader.setFeature("http://xml.org/sax/features/validation", false);
    	
        MSA_RDBDataHandler handler = new MSA_RDBDataHandler(_Database,td);

        reader.setContentHandler(handler);

		src = new InputSource(new FileInputStream(new File(fileName)));
        reader.parse(src);

        RDBToOnto.highPrintln(" (" + handler._Table.getNbLines() + " lines)");
          }
          catch (Exception e) {
                  System.err.println("RDBToOnto:  RDB Data Reader:parse: " + e.getMessage());
	              return false;
              }

          
          return true;
    }
        
       	public String getFileNameFromTableDefName(String tableName) {
       		String tabname = Utilities.replace(tableName,"_x0020_"," "); 
       		tabname = Utilities.replace(tabname," _x002F_ ","_"); 
       		tabname = Utilities.replace(tabname,"_x002F_","_"); 
    	return tabname;
    }
  
   
        
}
